precision highp float;
varying vec2 vTextureCoord;
uniform sampler2D sTexture;
uniform float surfaceWidth;
uniform float surfaceHeight;
#define x_y vec2(surfaceWidth, surfaceHeight)
uniform sampler2D eyeMaskTexture;
uniform vec2 outlinePoints[106];
uniform int notUseFilter;
uniform int enableChin;
uniform int enableEye;
uniform int enableNose;
uniform int enableFace;
uniform int enableForeHead;
uniform int enableCanthus;
uniform int enableCutFace;
#define RESHAPE_NUM 7
#define RESHAPE_INDEX_CHIN          0       //下巴
#define RESHAPE_INDEX_EYE           1       //眼睛
#define RESHAPE_INDEX_NOSE          2       //鼻子
#define RESHAPE_INDEX_FACE          3       //脸颊
#define RESHAPE_INDEX_FOREHEAD      4       //前额
#define RESHAPE_INDEX_CANTHUS       5       //眼角
#define RESHAPE_INDEX_CUT_FACE      6       //削脸
uniform float g_intensity[RESHAPE_NUM];
vec2 curCoord;
//#define ALTERNATIVE_FOR_ANDROID
#ifndef ALTERNATIVE_FOR_ANDROID
vec2 maskImageSize;
vec2 leftEyeMask;
vec2 rightEyeMask;
vec2 rotateFun(vec2 coord,vec2 circleCenter,vec2 maskCenter,float radius,float intensity)
{
float currentDistance = distance(coord,circleCenter);
float infect = currentDistance/radius;
if (infect<=1.0)
{
//for EyeMask.png
vec2 basicVec=(outlinePoints[77]- outlinePoints[74]);
vec2 normalVec=vec2(basicVec.y,-basicVec.x);
vec2 basicVec_Mask=(leftEyeMask- rightEyeMask);
vec2 normalVec_Mask=vec2(basicVec_Mask.y,-basicVec_Mask.x);
if(currentDistance<=radius )
{
vec2 maskCoord;
vec4 maskColor;
vec2 curVec=coord- circleCenter;
float e1=dot(curVec,basicVec)/(length(basicVec)*length(basicVec));
float e2=dot(curVec,normalVec)/(length(normalVec)*length(normalVec));
maskCoord=e1*basicVec_Mask+e2*normalVec_Mask+ maskCenter;
maskCoord=maskCoord/maskImageSize;
maskColor = texture2D(eyeMaskTexture,maskCoord);
infect=maskColor.a;
}
//旋转画面
vec2 rotateCenter=circleCenter;
//旋转角度
float rotateAngle=30.0;
rotateAngle=radians(rotateAngle)*intensity*infect;
float cos=cos(rotateAngle);
float sin=sin(rotateAngle);
if(distance(coord,outlinePoints[74])< radius)
sin=-sin;
//旋转矩阵
mat3 rotateMat=mat3(cos,-sin,0.0,
                    sin,cos,0.0,
                    0.0,0.0,1.0);
//计算偏移量矩阵
vec3 deltaOffset;
deltaOffset=rotateMat*vec3(coord.x- rotateCenter.x,coord.y- rotateCenter.y,1.0);
//当前坐标加上偏移量，即旋转后的坐标
coord.x=deltaOffset.x+rotateCenter.x;
coord.y=deltaOffset.y+rotateCenter.y;
}
return coord;
}
#endif
vec2 stretchFun(vec2 textureCoord, vec2 originPosition, vec2 targetPosition, float radius)
{
vec2 offset = vec2(0.0);
vec2 result = vec2(0.0);
vec2 direction = targetPosition - originPosition;
float lengthA = length(direction);
//    if(lengthA<0.0001)   return (textureCoord-direction);
float infect = distance(textureCoord, originPosition)/radius;
infect = 1.0-infect;
infect = clamp(infect,0.0,1.0);
offset = direction * infect;
result = textureCoord - offset;
return result;
}
vec2 enlargeFun(vec2 coord,vec2 circleCenter,float radius,float intensity)
{
float currentDistance = distance(coord,circleCenter);
if (currentDistance<=radius)
{
float weight = currentDistance/radius;
weight = 1.0-intensity*(1.0-weight*weight);
weight = clamp(weight,0.0,1.0);
coord = circleCenter+(coord-circleCenter)*weight;
}
return coord;
}
vec2 narrowFun(vec2 coord,vec2 circleCenter,float radius,float intensity)
{
float currentDistance = distance(coord,circleCenter);
if (currentDistance<=radius)
{
float weight = currentDistance/radius;
weight = 1.0-intensity*(1.0-weight*weight);
weight = clamp(weight,0.0001,1.0);
coord = circleCenter+(coord-circleCenter)/weight;
}
return coord;
}
void  processFace()
{
    vec2 realMouthCenter    = (outlinePoints[98]+outlinePoints[102])*0.5;
    vec2 vDirection         = (outlinePoints[16]-outlinePoints[46])/distance(outlinePoints[46],outlinePoints[16]);
    vec2 srcPoint           = vec2(0.0);
    vec2 dstPoint           = vec2(0.0);
    vec2 offset             = vec2(0.0);
    float radius = 0.0;
    float weight = 1.0;
    float currentDistance = 0.0;
    float irisWidth         = distance(outlinePoints[74],outlinePoints[77]);
    float standardLength    = irisWidth;
    float factor = 0.0;
    float level = 0.0;
    //标准脸
    float face_scale=(g_intensity[RESHAPE_INDEX_FACE]*0.5+0.25)*0.8;
    float eye_scale=g_intensity[RESHAPE_INDEX_EYE]*0.65+0.1-0.075;
    vec2 pChinCenter = outlinePoints[46]+(outlinePoints[16]-outlinePoints[46])*0.7;
    vec2 topHeadPoint = outlinePoints[46]+(outlinePoints[46]-outlinePoints[16])*1.9;
    //enlarge or narrow eyes
    if (enableEye == 1) {
        radius = standardLength * 0.35;//0.35
        curCoord = enlargeFun(curCoord,
                              outlinePoints[74] + (outlinePoints[77] - outlinePoints[74]) * 0.05,
                              radius, 0.105 * eye_scale);    //0.3
        curCoord = enlargeFun(curCoord,
                              outlinePoints[77] + (outlinePoints[74] - outlinePoints[77]) * 0.05,
                              radius, 0.105 * eye_scale);    //0.3
    }
    //stretch face
    if (enableFace == 1) {
        radius = standardLength;
        srcPoint = outlinePoints[3];
        dstPoint = srcPoint + (outlinePoints[44] - srcPoint) * 0.065 * face_scale;
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        srcPoint = outlinePoints[29];
        dstPoint = srcPoint + (outlinePoints[44] - srcPoint) * 0.065 * face_scale;
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        radius = standardLength * 1.1;
        srcPoint = outlinePoints[10];
        dstPoint = srcPoint + (outlinePoints[46] - srcPoint) * 0.06 * face_scale;//0.06
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        srcPoint = outlinePoints[22];
        dstPoint = srcPoint + (outlinePoints[46] - srcPoint) * 0.06 * face_scale;//0.06
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        srcPoint = outlinePoints[13];
        dstPoint = srcPoint + (pChinCenter - srcPoint) * 0.1 * face_scale;//0.1
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        srcPoint = outlinePoints[19];
        dstPoint = srcPoint + (pChinCenter - srcPoint) * 0.1 * face_scale;//0.1
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        radius = standardLength * 1.35;
        srcPoint = outlinePoints[16];
        dstPoint = srcPoint + (outlinePoints[46] - srcPoint) * 0.08 * face_scale;//0.08
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        radius = standardLength * 0.6;//0.6
        pChinCenter = outlinePoints[46] + (outlinePoints[16] - outlinePoints[46]) * 0.5;
        srcPoint = outlinePoints[13];
        dstPoint = srcPoint + (pChinCenter - curCoord) * 0.05 * face_scale;//0.07
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        srcPoint = outlinePoints[19];
        dstPoint = srcPoint + (pChinCenter - curCoord) * 0.05 * face_scale;//0.07
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        srcPoint = outlinePoints[10];
        dstPoint =
                srcPoint + (outlinePoints[46] - curCoord) * 0.012 * face_scale;//0.035   ->20171208
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        srcPoint = outlinePoints[22];
        dstPoint =
                srcPoint + (outlinePoints[46] - curCoord) * 0.012 * face_scale;//0.035   ->20171208
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        srcPoint = outlinePoints[8];
        dstPoint = srcPoint + (outlinePoints[46] - curCoord) * 0.02 * face_scale;//0.015  ->20171208
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        srcPoint = outlinePoints[24];
        dstPoint =
                srcPoint + (outlinePoints[46] - curCoord) * 0.02 * face_scale;//0.015   ->20171208
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        radius = standardLength * 0.35;//0.35
        srcPoint = outlinePoints[16];
        dstPoint = srcPoint - (pChinCenter - srcPoint) * 0.05 * face_scale;//0.03
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
    }
    //五官微调
    //cut face
    if (enableCutFace == 1) {
        vec2 realHeadLeft = outlinePoints[0]+(outlinePoints[0]-outlinePoints[1])*3.0;
        vec2 realHeadRight = outlinePoints[32]+(outlinePoints[32]-outlinePoints[31])*3.0;
        //process left face
        factor = g_intensity[RESHAPE_INDEX_CUT_FACE];
        radius = standardLength * 0.8;
        srcPoint = realHeadLeft + (realHeadLeft - outlinePoints[43]) * 0.15;//0.2
        dstPoint = srcPoint + (srcPoint - outlinePoints[43]) * factor;
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        srcPoint = outlinePoints[29] + (outlinePoints[29] - outlinePoints[43]) * 0.1;//0.2
        dstPoint = srcPoint + (srcPoint - outlinePoints[43]) * factor;
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        srcPoint = outlinePoints[24] + (outlinePoints[24] - outlinePoints[46]) * 0.3;//0.2
        dstPoint = srcPoint + (srcPoint - outlinePoints[46]) * factor;
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        srcPoint = outlinePoints[21] + (outlinePoints[21] - outlinePoints[46]) * 0.2;
        dstPoint = srcPoint + (srcPoint - outlinePoints[46]) * factor;
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        //process right face
        srcPoint = realHeadRight + (realHeadRight - outlinePoints[43]) * 0.15;//0.2
        dstPoint = srcPoint + (srcPoint - outlinePoints[43]) * factor;
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        srcPoint = outlinePoints[3] + (outlinePoints[3] - outlinePoints[43]) * 0.1;//0.2
        dstPoint = srcPoint + (srcPoint - outlinePoints[43]) * factor;
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        srcPoint = outlinePoints[8] + (outlinePoints[8] - outlinePoints[46]) * 0.3;//0.2
        dstPoint = srcPoint + (srcPoint - outlinePoints[46]) * factor;
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        srcPoint = outlinePoints[11] + (outlinePoints[11] - outlinePoints[46]) * 0.2;
        dstPoint = srcPoint + (srcPoint - outlinePoints[46]) * factor;
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
    }
    //process nose
    if (enableNose == 1) {
        level = g_intensity[RESHAPE_INDEX_NOSE];
        radius = standardLength * 0.4;//0.3
        factor = level * 0.08;//0.1
        srcPoint = outlinePoints[82];
        dstPoint = srcPoint + (outlinePoints[83] - srcPoint) * factor;
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        srcPoint = outlinePoints[83];
        dstPoint = srcPoint + (outlinePoints[82] - srcPoint) * factor;
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        // srcPoint=outlinePoints[80];
        srcPoint = outlinePoints[80] + (outlinePoints[78] - outlinePoints[80]) * 0.5;
        dstPoint = srcPoint + (outlinePoints[81] - srcPoint) * factor;
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        // srcPoint=outlinePoints[81];
        srcPoint = outlinePoints[81] + (outlinePoints[79] - outlinePoints[81]) * 0.5;
        dstPoint = srcPoint + (outlinePoints[80] - srcPoint) * factor;
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
    }
#ifndef ALTERNATIVE_FOR_ANDROID
    //rotate eyes
    if (enableCanthus == 1) {
        maskImageSize = vec2(1113.0, 954.0);
        leftEyeMask = vec2(374.0, 238.0);
        rightEyeMask = vec2(732.0, 244.0);
        // vec2 temp=leftEyeMask;
        // leftEyeMask=rightEyeMask;
        // rightEyeMask=temp;
        level = g_intensity[RESHAPE_INDEX_CANTHUS];
        factor = level * 0.135;//0.1
        radius = standardLength * 0.3;//0.4
        srcPoint = outlinePoints[74];
        curCoord = rotateFun(curCoord, srcPoint, leftEyeMask, radius, factor);
        srcPoint = outlinePoints[77];
        curCoord = rotateFun(curCoord, srcPoint, rightEyeMask, radius, factor);
    }
    //process chin
    if (enableChin == 1) {
        level = g_intensity[RESHAPE_INDEX_CHIN];//1.0
        vec2 chin_center = outlinePoints[16] + (outlinePoints[16] - realMouthCenter) /
                                               distance(outlinePoints[16], realMouthCenter) *
                                               standardLength * 0.5;
        vec2 chin_left = chin_center + (outlinePoints[11] - outlinePoints[21]) * 0.45;
        vec2 chin_right = chin_center + (outlinePoints[21] - outlinePoints[11]) * 0.45;
        radius = standardLength * 0.9;
        srcPoint = chin_center;
        dstPoint = srcPoint + vDirection * standardLength * 0.105 * level;
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        radius = standardLength;
        srcPoint = chin_left;
        dstPoint = srcPoint + (srcPoint - outlinePoints[43]) * 0.0768 *
                              level;//srcPoint+vDirection*standardLength*0.125;
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        srcPoint = chin_right;
        dstPoint = srcPoint + (srcPoint - outlinePoints[43]) * 0.0768 *
                              level;//srcPoint+vDirection*standardLength*0.125;
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
    }
    //process forehead
    if (enableForeHead == 1) {
        level = g_intensity[RESHAPE_INDEX_FOREHEAD];
        vec2 foreHead_Mid = outlinePoints[43] + (outlinePoints[43] - outlinePoints[49]) * 1.8;
        vec2 foreHead_left = foreHead_Mid + (outlinePoints[74] - outlinePoints[77]) * 0.8;
        vec2 foreHead_right = foreHead_Mid + (outlinePoints[77] - outlinePoints[74]) * 0.8;
        radius = standardLength * 1.0;
        srcPoint = foreHead_Mid;
        dstPoint = srcPoint + (outlinePoints[43] - outlinePoints[49]) * 0.08 * level;
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        srcPoint = foreHead_left;
        dstPoint = srcPoint + (outlinePoints[43] - outlinePoints[49]) * 0.1 * level;//0.07
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
        srcPoint = foreHead_right;
        dstPoint = srcPoint + (outlinePoints[43] - outlinePoints[49]) * 0.1 * level;//0.07
        curCoord = stretchFun(curCoord, srcPoint, dstPoint, radius);
    }
#endif
}
void main(void)
{
    if(0==notUseFilter){
        curCoord = vTextureCoord*x_y;
        processFace();
        curCoord = curCoord/x_y;
    }else{
        curCoord = vTextureCoord;
    }
    gl_FragColor = texture2D(sTexture, curCoord);
}
